@@ -27,7 +27,7 @@
  *  Created by Shantonu Sen <ssen@apple.com> on Thu Dec 6 2001.
  *  Copyright (c) 2001-2005 Apple Computer, Inc. All rights reserved.
  *
- *  $Id: handleFolder.c,v 1.2 2006/01/27 22:57:33 ssen Exp $
+ *  $Id: handleFolder.c,v 1.79 2006/07/21 14:59:24 ssen Exp $
  *
  */
 
@@ -45,20 +45,14 @@
 
 #include "bless.h"
 #include "bless_private.h"
+#include "protos.h"
 
 enum {
   kIsInvisible                  = 0x4000, /* Files and folders */
 };
 
-extern int blesscontextprintf(BLContextPtr context, int loglevel, char const *fmt, ...)
-    __attribute__ ((format (printf, 3, 4)));
-
 
 static int isOFLabel(const char *data, int labelsize);
-extern int setboot(BLContextPtr context, char *device, CFDataRef bootxData,
-				   CFDataRef labelData);
-extern int setefifilepath(BLContextPtr context, const char * path, int bootNext,
-				   int bootLegacy, const char *optionalData);
 
 int modeFolder(BLContextPtr context, struct clarg actargs[klast]) {
 	
@@ -363,8 +357,14 @@ int modeFolder(BLContextPtr context, struct clarg actargs[klast]) {
         // of HFSX. These filesystems we don't want blessed, because we don't
         // want future versions of OF to list them as bootable, but rather
         // prefer the Apple_Boot partition
+        //
+        // For EFI-based systems, it's OK to set finderinfo[0], and indeed
+        // a better user experience so that the EFI label shows up
         
-        if((sb.f_reserved1 & ~1) && (getenv("BL_OVERRIDE_BLESS_HFSX") == NULL)) {
+        if(actargs[ksetboot].present &&
+           (preboot == kBLPreBootEnvType_OpenFirmware) &&
+           (sb.f_reserved1 & ~1)
+           ) {
             blesscontextprintf(context, kBLLogLevelVerbose,  "%s is not HFS+ or Journaled HFS+. Not setting finderinfo[0]...\n", actargs[kmount].argument );
             oldwords[0] = 0;
         } else {
@@ -447,7 +447,7 @@ int modeFolder(BLContextPtr context, struct clarg actargs[klast]) {
 		if(actargs[kfolder].present) {
 			char sysfolder[MAXPATHLEN];
 		
-			sprintf(sysfolder, "%s/Volume Name Icon", actargs[kfolder].argument);
+			sprintf(sysfolder, "%s/.disk_label", actargs[kfolder].argument);
 			
 			blesscontextprintf(context, kBLLogLevelVerbose,  "Putting label bitmap in %s\n",
 							   sysfolder );
@@ -480,13 +480,16 @@ int modeFolder(BLContextPtr context, struct clarg actargs[klast]) {
 			// if you blessed the volume, then just point EFI at the volume.
 			// only if you didn't bless, but you have something interesting
 			// to point at, should you use actargs[kfile]
+            
 
             err = setefifilepath(context, ( !shouldBless && actargs[kfile].present ?
 											actargs[kfile].argument :
 											actargs[kmount].argument),
                                  actargs[knextonly].present,
                                  actargs[klegacy].present,
-                                 actargs[koptions].present ? actargs[koptions].argument : NULL);
+                                 actargs[klegacydrivehint].present ? actargs[klegacydrivehint].argument : NULL,
+                                 actargs[koptions].present ? actargs[koptions].argument : NULL,
+                                 actargs[kshortform].present ? true : false);
             if(err) {
                 return 3;
             }
