@@ -27,7 +27,7 @@
  *  Created by Shantonu Sen <ssen@apple.com> on Mon Aug 25 2003.
  *  Copyright (c) 2003-2005 Apple Computer, Inc. All rights reserved.
  *
- *  $Id: minibless.c,v 1.2 2006/01/27 22:57:33 ssen Exp $
+ *  $Id: minibless.c,v 1.12 2006/07/18 22:09:51 ssen Exp $
  *
  */
 
@@ -41,10 +41,9 @@
 #include <err.h>
 
 #include "bless.h"
+#include "protos.h"
 
 void usage(char *program);
-extern int setefidevice(BLContextPtr context, const char * bsdname, int bootNext,
-				 int bootLegacy, const char *optionalData);
 
 int main(int argc, char *argv[]) {
 	
@@ -80,14 +79,16 @@ int main(int argc, char *argv[]) {
 #else
 #ifdef __i386__
 	if(0 != setefidevice(NULL, device + 5 /* strlen("/dev/") */,
-					   0,
-			                   0,
-					   NULL)) {
+			     0,
+			     0,
+			     NULL,
+			     NULL,
+                       false)) {
 		errx(1, "Can't set EFI");		
 	}
 	
 #else
-#err wha?????
+#error wha?????
 #endif
 #endif
 	
